@@ -130,16 +130,13 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 	if (must_be_container(dfd)) {
 		if (!st)
 			st = super_by_fd(dfd, NULL);
-		if (st)
-			st->ignore_hw_compat = 1;
 		if (st && st->ss->load_container)
 			rv = st->ss->load_container(st, dfd, NULL);
 
 		close(dfd);
 		if (!rv && st->ss->container_content) {
 			if (map_lock(&map))
-				pr_err("failed to get "
-				       "exclusive lock on mapfile\n");
+				pr_err("failed to get exclusive lock on mapfile\n");
 			if (c->export)
 				printf("MD_DEVNAME=%s\n", devname);
 			rv = Incremental_container(st, devname, c, NULL);
@@ -196,18 +193,19 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 	policy = disk_policy(&dinfo);
 	have_target = policy_check_path(&dinfo, &target_array);
 
-	if (st == NULL && (st = guess_super(dfd)) == NULL) {
+	if (st == NULL && (st = guess_super_type(dfd, guess_array)) == NULL) {
 		if (c->verbose >= 0)
 			pr_err("no recognisable superblock on %s.\n",
 			       devname);
 		rv = try_spare(devname, &dfd, policy,
 			       have_target ? &target_array : NULL,
-			       st, c->verbose);
+			       NULL, c->verbose);
 		goto out;
 	}
-	st->ignore_hw_compat = 1;
+	st->ignore_hw_compat = 0;
+
 	if (st->ss->compare_super == NULL ||
-	    st->ss->load_super(st, dfd, NULL)) {
+	    st->ss->load_super(st, dfd, c->verbose >= 0 ? devname : NULL)) {
 		if (c->verbose >= 0)
 			pr_err("no RAID superblock on %s.\n",
 				devname);
@@ -229,8 +227,7 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 	if (match && match->devname
 	    && strcasecmp(match->devname, "<ignore>") == 0) {
 		if (c->verbose >= 0)
-			pr_err("array containing %s is explicitly"
-				" ignored by mdadm.conf\n",
+			pr_err("array containing %s is explicitly ignored by mdadm.conf\n",
 				devname);
 		goto out;
 	}
@@ -251,8 +248,7 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 	if (!match && !conf_test_metadata(st->ss->name, policy,
 					  (trustworthy == LOCAL))) {
 		if (c->verbose >= 1)
-			pr_err("%s has metadata type %s for which "
-			       "auto-assembly is disabled\n",
+			pr_err("%s has metadata type %s for which auto-assembly is disabled\n",
 			       devname, st->ss->name);
 		goto out;
 	}
@@ -289,8 +285,7 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 	/* 4/ Check if array exists.
 	 */
 	if (map_lock(&map))
-		pr_err("failed to get exclusive lock on "
-			"mapfile\n");
+		pr_err("failed to get exclusive lock on mapfile\n");
 	/* Now check we can get O_EXCL.  If not, probably "mdadm -A" has
 	 * taken over
 	 */
@@ -352,8 +347,7 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 			 * So reject it.
 			 */
 			ioctl(mdfd, STOP_ARRAY, NULL);
-			pr_err("You have an old buggy kernel which cannot support\n"
-			       "      --incremental reliably.  Aborting.\n");
+			pr_err("You have an old buggy kernel which cannot support\n      --incremental reliably.  Aborting.\n");
 			rv = 2;
 			goto out_unlock;
 		}
@@ -420,8 +414,7 @@ int Incremental(struct mddev_dev *devlist, struct context *c,
 			st2 = dup_super(st);
 			if (st2->ss->load_super(st2, dfd2, NULL) ||
 			    st->ss->compare_super(st, st2) != 0) {
-				pr_err("metadata mismatch between %s and "
-				       "chosen array %s\n",
+				pr_err("metadata mismatch between %s and chosen array %s\n",
 				       devname, chosen_name);
 				close(dfd2);
 				rv = 2;
@@ -864,8 +857,7 @@ static int array_try_spare(char *devname, int *dfdp, struct dev_policy *pol,
 	 */
 
 	if (map_lock(&map)) {
-		pr_err("failed to get exclusive lock on "
-			"mapfile\n");
+		pr_err("failed to get exclusive lock on mapfile\n");
 		return 1;
 	}
 	for (mp = map ; mp ; mp = mp->next) {
@@ -911,8 +903,7 @@ static int array_try_spare(char *devname, int *dfdp, struct dev_policy *pol,
 					sra->text_version);
 			if (!st2) {
 				if (verbose > 1)
-					pr_err("not adding %s to %s"
-						" as metadata not recognised.\n",
+					pr_err("not adding %s to %s as metadata not recognised.\n",
 						devname, mp->path);
 				goto next;
 			}
@@ -994,8 +985,7 @@ static int array_try_spare(char *devname, int *dfdp, struct dev_policy *pol,
 		if (domain_test(dl, pol, st2->ss->name) != 1) {
 			/* domain test fails */
 			if (verbose > 1)
-				pr_err("not adding %s to %s as"
-					" it is not in a compatible domain\n",
+				pr_err("not adding %s to %s as it is not in a compatible domain\n",
 					devname, mp->path);
 
 			goto next;
@@ -1132,6 +1122,7 @@ static int partition_try_spare(char *devname, int *dfdp, struct dev_policy *pol,
 		if (st2 == NULL ||
 		    st2->ss->load_super(st2, fd, NULL) < 0)
 			goto next;
+		st2->ignore_hw_compat = 0;
 
 		if (!st) {
 			/* Check domain policy again, this time referring to metadata */
@@ -1267,8 +1258,7 @@ static int try_spare(char *devname, int *dfdp, struct dev_policy *pol,
 		    !policy_action_allows(pol, st?st->ss->name:NULL,
 					  act_spare_same_slot)) {
 			if (verbose > 1)
-				pr_err("%s is not bare, so not "
-					"considering as a spare\n",
+				pr_err("%s is not bare, so not considering as a spare\n",
 					devname);
 			return 1;
 		}
@@ -1359,12 +1349,11 @@ restart:
 			struct supertype *st = super_by_fd(mdfd, NULL);
 			int ret = 0;
 			struct map_ent *map = NULL;
-			if (st)
-				st->ignore_hw_compat = 1;
+
 			if (st && st->ss->load_container)
 				ret = st->ss->load_container(st, mdfd, NULL);
 			close(mdfd);
-			if (!ret && st->ss->container_content) {
+			if (!ret && st && st->ss->container_content) {
 				if (map_lock(&map))
 					pr_err("failed to get exclusive lock on mapfile\n");
 				ret = Incremental_container(st, me->path, c, only);
@@ -1576,8 +1565,7 @@ static int Incremental_container(struct supertype *st, char *devname,
 			if (match && match->devname &&
 			    strcasecmp(match->devname, "<ignore>") == 0) {
 				if (c->verbose > 0)
-					pr_err("array %s/%s is "
-					       "explicitly ignored by mdadm.conf\n",
+					pr_err("array %s/%s is explicitly ignored by mdadm.conf\n",
 					       match->container, match->member);
 				continue;
 			}
@@ -1710,34 +1698,34 @@ int IncrementalRemove(char *devname, char *id_path, int verbose)
 	char buf[32];
 
 	if (!id_path)
-		dprintf(Name ": incremental removal without --path <id_path> "
-			"lacks the possibility to re-add new device in this "
-			"port\n");
+		dprintf("incremental removal without --path <id_path> lacks the possibility to re-add new device in this port\n");
 
 	if (strchr(devname, '/')) {
-		pr_err("incremental removal requires a "
-			"kernel device name, not a file: %s\n", devname);
+		pr_err("incremental removal requires a kernel device name, not a file: %s\n", devname);
 		return 1;
 	}
 	ent = mdstat_by_component(devname);
 	if (!ent) {
 		if (verbose >= 0)
-			pr_err("%s does not appear to be a component "
-			       "of any array\n", devname);
+			pr_err("%s does not appear to be a component of any array\n", devname);
 		return 1;
 	}
 	sysfs_init(&mdi, -1, ent->devnm);
-	if (sysfs_get_str(&mdi, NULL, "array_state",
-			  buf, sizeof(buf)) > 0) {
-		if (strncmp(buf, "active", 6) == 0 ||
-		    strncmp(buf, "clean", 5) == 0)
-			sysfs_set_str(&mdi, NULL,
-				      "array_state", "read-auto");
+	mdfd = open_dev_excl(ent->devnm);
+	if (mdfd > 0) {
+		close(mdfd);
+		if (sysfs_get_str(&mdi, NULL, "array_state",
+				  buf, sizeof(buf)) > 0) {
+			if (strncmp(buf, "active", 6) == 0 ||
+			    strncmp(buf, "clean", 5) == 0)
+				sysfs_set_str(&mdi, NULL,
+					      "array_state", "read-auto");
+		}
 	}
 	mdfd = open_dev(ent->devnm);
 	if (mdfd < 0) {
 		if (verbose >= 0)
-			pr_err("Cannot open array %s!!\n", ent->dev);
+			pr_err("Cannot open array %s!!\n", ent->devnm);
 		free_mdstat(ent);
 		return 1;
 	}
@@ -1759,11 +1747,11 @@ int IncrementalRemove(char *devname, char *id_path, int verbose)
 		struct mdstat_ent *mdstat = mdstat_read(0, 0);
 		struct mdstat_ent *memb;
 		for (memb = mdstat ; memb ; memb = memb->next)
-			if (is_container_member(memb, ent->dev)) {
+			if (is_container_member(memb, ent->devnm)) {
 				int subfd = open_dev(memb->devnm);
 				if (subfd >= 0) {
 					rv |= Manage_subdevs(
-						memb->dev, subfd,
+						memb->devnm, subfd,
 						&devlist, verbose, 0,
 						NULL, 0);
 					close(subfd);
@@ -1771,7 +1759,7 @@ int IncrementalRemove(char *devname, char *id_path, int verbose)
 			}
 		free_mdstat(mdstat);
 	} else
-		rv |= Manage_subdevs(ent->dev, mdfd, &devlist,
+		rv |= Manage_subdevs(ent->devnm, mdfd, &devlist,
 				    verbose, 0, NULL, 0);
 	if (rv & 2) {
 		/* Failed due to EBUSY, try to stop the array.
@@ -1779,7 +1767,7 @@ int IncrementalRemove(char *devname, char *id_path, int verbose)
 		 */
 		int devid = devnm2devid(ent->devnm);
 		run_udisks("--unmount", map_dev(major(devid),minor(devid), 0));
-		rv = Manage_stop(ent->dev, mdfd, verbose, 1);
+		rv = Manage_stop(ent->devnm, mdfd, verbose, 1);
 		if (rv)
 			/* At least we can try to trigger a 'remove' */
 			sysfs_uevent(&mdi, "remove");
@@ -1789,7 +1777,7 @@ int IncrementalRemove(char *devname, char *id_path, int verbose)
 		}
 	} else {
 		devlist.disposition = 'r';
-		rv = Manage_subdevs(ent->dev, mdfd, &devlist,
+		rv = Manage_subdevs(ent->devnm, mdfd, &devlist,
 				    verbose, 0, NULL, 0);
 	}
 	close(mdfd);
