@@ -20,7 +20,7 @@
 
 #include "mdadm.h"
 
-inline void sb_le_to_cpu(bitmap_super_t *sb)
+static inline void sb_le_to_cpu(bitmap_super_t *sb)
 {
 	sb->magic = __le32_to_cpu(sb->magic);
 	sb->version = __le32_to_cpu(sb->version);
@@ -34,7 +34,7 @@ inline void sb_le_to_cpu(bitmap_super_t *sb)
 	sb->write_behind = __le32_to_cpu(sb->write_behind);
 }
 
-inline void sb_cpu_to_le(bitmap_super_t *sb)
+static inline void sb_cpu_to_le(bitmap_super_t *sb)
 {
 	sb_le_to_cpu(sb); /* these are really the same thing */
 }
@@ -74,7 +74,7 @@ typedef struct bitmap_info_s {
 } bitmap_info_t;
 
 /* count the dirty bits in the first num_bits of byte */
-inline int count_dirty_bits_byte(char byte, int num_bits)
+static inline int count_dirty_bits_byte(char byte, int num_bits)
 {
 	int num = 0;
 
@@ -140,8 +140,7 @@ bitmap_info_t *bitmap_fd_read(int fd, int brief)
 	info = xmalloc(sizeof(*info));
 
 	if (n < sizeof(info->sb)) {
-		pr_err("failed to read superblock of bitmap "
-		       "file: %s\n", strerror(errno));
+		pr_err("failed to read superblock of bitmap file: %s\n", strerror(errno));
 		free(info);
 		free(buf);
 		return NULL;
@@ -182,8 +181,7 @@ bitmap_info_t *bitmap_fd_read(int fd, int brief)
 	}
 
 	if (read_bits < total_bits) { /* file truncated... */
-		pr_err("WARNING: bitmap file is not large "
-			"enough for array size %llu!\n\n",
+		pr_err("WARNING: bitmap file is not large enough for array size %llu!\n\n",
 			(unsigned long long)info->sb.sync_size);
 		total_bits = read_bits;
 	}
