@@ -490,18 +490,8 @@ extern int map_update(struct map_ent **mpp, char *devnm, char *metadata,
 		      int uuid[4], char *path);
 extern void map_remove(struct map_ent **map, char *devnm);
 extern struct map_ent *map_by_uuid(struct map_ent **map, int uuid[4]);
-#ifdef MDASSEMBLE
-static inline struct map_ent *map_by_devnm(struct map_ent **map, char *name)
-{
-	return NULL;
-}
-static inline void map_free(struct map_ent *map)
-{
-}
-#else
 extern struct map_ent *map_by_devnm(struct map_ent **map, char *devnm);
 extern void map_free(struct map_ent *map);
-#endif
 extern struct map_ent *map_by_name(struct map_ent **map, char *name);
 extern void map_read(struct map_ent **melp);
 extern int map_write(struct map_ent *mel);
